<?php
/*
 * Omnistream Events Widget Class
*/
// Block direct requests
if (!defined('ABSPATH'))
  die('-1');

/**
 * Adds Omnistream_Events widget.
 */
class Omnistream_Events extends WP_Widget {
  /**
   * Register widget with WordPress.
   */
  function __construct() {
    parent::__construct(
      'omnistream_events', // Base ID
      __('Omnistream Events Widget', 'text_domain'), // Name
      array('description' => __('Show upcoming events', 'text_domain'),) // Args
    );
  }
  /**
   * Front-end display of widget.
   *
   * @see WP_Widget::widget()
   *
   * @param array $args     Widget arguments.
   * @param array $instance Saved values from database.
   */
  public function widget($args, $instance) {
    global $post;

    extract($args);

    // set defaults
    $instance = wp_parse_args((array) $instance, $this->events_defaults());
    $title = apply_filters('widget_title', $instance['title']);

    echo $before_widget;

    if ($title) {
      echo $before_title . $title . $after_title;
    }

    $event_category = $instance['default_event_category'];
    $event_page = $instance['default_event_page'];

    if (is_single()) {
      $categories = get_the_category();
      $post_categories = array();

      // get a list of categories and all parents
      foreach ($categories as $cat) {
        $post_categories[] = $cat->cat_ID;
        $post_categories = array_merge($post_categories, get_ancestors($cat->cat_ID, 'category'));
      }

      // find if any categories are related to event categories
      if (in_array($instance['event_category_1'], $post_categories)) {
        $event_category = $instance['event_category_1'];
        $event_page = $instance['event_page_1'];
      } elseif (in_array($instance['event_category_2'], $post_categories)) {
        $event_category = $instance['event_category_2'];
        $event_page = $instance['event_page_2'];
      }
    } elseif (!is_front_page()) {
      // we must be on a category page
      $categories = get_the_category();
      if (empty($categories)) {
        $page_category = get_category_by_slug($post->post_name);
        if ($page_category) {
          $page_cat_id = $page_category->cat_ID;
        }
      } else {
        $page_cat_id = $categories[0]->cat_ID;
      }

      if (isset($page_cat_id)) {
        $page_categories[] = $page_cat_id;
        $page_categories = array_merge($page_categories, get_ancestors($page_cat_id, 'category'));

        if (in_array($instance['event_category_1'], $page_categories)) {
          $event_category = $instance['event_category_1'];
          $event_page = $instance['event_page_1'];
        } elseif (in_array($instance['event_category_2'], $page_categories)) {
          $event_category = $instance['event_category_2'];
          $event_page = $instance['event_page_2'];
        }
      }
    }

    $events_count = $instance['events_count'];

    if ($event_category == -1) {
      $events_array = array();
    } else {
      $args = array(
        'post_type' => 'omnistream-event',
        'category' => $event_category,
        'meta_query' => array(
          'relation' => 'AND',
          array(
            'key' => 'OmnistreamEventStartDate',
            'value' => current_time("Y-m-d"),
            'compare' => '>='
          ),
          array(
            'relation' => 'OR',
            array(
              'key' => 'OmnistreamPostExpiry',
              'value' => gmdate("Y-m-d\TH:i:s\Z"),
              'compare' => '>'
            ),
            array(
              'key' => 'OmnistreamPostExpiry',
              'compare' => 'NOT EXISTS'
            )
          )
        ),
        'meta_key' => 'OmnistreamEventStartDate',
        'orderby' => 'meta_value',
        'order'   => 'ASC',
        'post_status' => 'publish',
        'posts_per_page' => $events_count
      );
      $events_array = get_posts($args);
    }

    echo '<div class="omnistream-events">';

    if (count($events_array) == 0) {
      echo 'No Events';
    } else {
      foreach ($events_array as $event) {
        $start_date_Z = get_post_meta($event->ID, 'OmnistreamEventStartDate', true);
        $end_date_Z = get_post_meta($event->ID, 'OmnistreamEventEndDate', true);
        $start_month = date('M', strtotime($start_date_Z));
        $end_month = date('M', strtotime($end_date_Z));
        $start_date_string = date('M d', strtotime($start_date_Z));
        $end_date_string = date('M d', strtotime($end_date_Z));
        $end_day = date('d', strtotime($end_date_Z));
        $city = get_post_meta($event->ID, 'OmnistreamEventCity', true);
        $state = get_post_meta($event->ID, 'OmnistreamEventState', true);
        $permalink = get_post_permalink($event->ID);

        echo '<a class="event-line" href="' . $permalink . '">';
        echo '<div class="event-description">&bull;&nbsp;<span class="event-date">';
        if ($start_date_string == $end_date_string) {
          echo $start_date_string;
        }
        elseif ($start_month == $end_month) {
          echo $start_date_string . '-' . $end_day;
        }
        else {
          echo $start_date_string . ' - ' . $end_date_string;
        }
        echo '</span>:&nbsp;';
        echo $event->post_title . '</div><div class="event-location">&nbsp;-&nbsp;' . $city . ',&nbsp;' . $state . '</div>';
        echo '</a>';
      }
    }

    if ($event_page != "") {
      $event_page_link = get_permalink($event_page);
      echo '<a href="' . $event_page_link . '"><button type="button">View All Events</button></a>';
    }

    echo '</div>';

    echo $after_widget;
  }
  /**
   * Back-end widget form.
   *
   * @see WP_Widget::form()
   *
   * @param array $instance Previously saved values from database.
   */
  public function form($instance) {

    // Set default values
    $instance = wp_parse_args((array) $instance, $this->events_defaults());

    $title = esc_attr($instance['title']);
    $events_count = esc_attr($instance['events_count']);
    $event_category_1 = esc_attr($instance['event_category_1']);
    $event_page_1 = esc_attr($instance['event_page_1']);
    $event_category_2 = esc_attr($instance['event_category_2']);
    $event_page_2 = esc_attr($instance['event_page_2']);
    $default_event_category = esc_attr($instance['default_event_category']);
    $default_event_page = esc_attr($instance['default_event_page']);
    ?>

    <p>
      <label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?></label>
      <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>">
    </p>
    <p>
      <label for="<?php echo $this->get_field_id('events_count'); ?>"><?php _e('Number of events to display:'); ?></label>
      <input class="widefat" id="<?php echo $this->get_field_id('events_count'); ?>" name="<?php echo $this->get_field_name('events_count'); ?>" type="number" value="<?php echo esc_attr($events_count); ?>">
    </p>
    <p>
      <label for="<?php echo $this->get_field_id('event_category_1'); ?>"><?php _e('First optional event category (list these events when on pages related to this category):'); ?></label>
      <?php
        $args = array(
          'name'              => $this->get_field_name('event_category_1'),
          'id'                => $this->get_field_id('event_category_1'),
          'selected'          => $event_category_1,
          'orderby'           => 'name',
          'hide_empty'        => false,
          'show_option_none'  => 'no category',
          'hierarchical'      => true,
          'depth'             => 1
        );
        wp_dropdown_categories($args);
      ?>
    </p>
    <p>
      <label for="<?php echo $this->get_field_id('event_page_1'); ?>"><?php _e('Page listing these events:'); ?></label>
      <?php
        $args = array(
          'name'              => $this->get_field_name('event_page_1'),
          'id'                => $this->get_field_id('event_page_1'),
          'selected'          => $event_page_1,
          'orderby'           => 'name',
          'hide_empty'        => false,
          'show_option_none'  => 'no page'
        );
        wp_dropdown_pages($args);
      ?>
    </p>
    <p>
      <label for="<?php echo $this->get_field_id('event_category_2'); ?>"><?php _e('Second optional event category (list these events when on pages related to this category):'); ?></label>
      <?php
        $args = array(
          'name'              => $this->get_field_name('event_category_2'),
          'id'                => $this->get_field_id('event_category_2'),
          'selected'          => $event_category_2,
          'orderby'           => 'name',
          'hide_empty'        => false,
          'show_option_none'  => 'no category',
          'hierarchical'      => true,
          'depth'             => 1
        );
        wp_dropdown_categories($args);
      ?>
    </p>
    <p>
      <label for="<?php echo $this->get_field_id('event_page_2'); ?>"><?php _e('Page listing these events:'); ?></label>
      <?php
        $args = array(
          'name'              => $this->get_field_name('event_page_2'),
          'id'                => $this->get_field_id('event_page_2'),
          'selected'          => $event_page_2,
          'orderby'           => 'name',
          'hide_empty'        => false,
          'show_option_none'  => 'no page'
        );
        wp_dropdown_pages($args);
      ?>
    </p>
    <p>
      <label for="<?php echo $this->get_field_id('default_event_category'); ?>"><?php _e('Default category (list these events when on home page, on pages not related to any event category, or when no event categories are set:'); ?></label>
      <?php
        $args = array(
          'name'              => $this->get_field_name('default_event_category'),
          'id'                => $this->get_field_id('default_event_category'),
          'selected'          => $default_event_category,
          'orderby'           => 'name',
          'hide_empty'        => false,
          'show_option_none'  => 'no category',
          'hierarchical'      => true,
          'depth'              => 1
        );
        wp_dropdown_categories($args);
      ?>
    </p>
        <p>
      <label for="<?php echo $this->get_field_id('default_event_page'); ?>"><?php _e('Page listing these events:'); ?></label>
      <?php
        $args = array(
          'name'              => $this->get_field_name('default_event_page'),
          'id'                => $this->get_field_id('default_event_page'),
          'selected'          => $default_event_page,
          'orderby'           => 'name',
          'hide_empty'        => false,
          'show_option_none'  => 'no page'
        );
        wp_dropdown_pages($args);
      ?>
    </p>
    <?php
  }
  /**
   * Sanitize widget form values as they are saved.
   *
   * @see WP_Widget::update()
   *
   * @param array $new_instance Values just sent to be saved.
   * @param array $old_instance Previously saved values from database.
   *
   * @return array Updated safe values to be saved.
   */
  public function update($new_instance, $old_instance) {
    $instance = array();
    $instance['title'] = (! empty($new_instance['title'])) ? strip_tags($new_instance['title']) : '';
    $instance['events_count'] = (! empty($new_instance['events_count'])) ? strip_tags($new_instance['events_count']) : '';
    $instance['event_category_1'] = (! empty($new_instance['event_category_1'])) ? strip_tags($new_instance['event_category_1']) : '';
    $instance['event_page_1'] = (! empty($new_instance['event_page_1'])) ? strip_tags($new_instance['event_page_1']) : '';
    $instance['event_category_2'] = (! empty($new_instance['event_category_2'])) ? strip_tags($new_instance['event_category_2']) : '';
    $instance['event_page_2'] = (! empty($new_instance['event_page_2'])) ? strip_tags($new_instance['event_page_2']) : '';
    $instance['default_event_category'] = (! empty($new_instance['default_event_category'])) ? strip_tags($new_instance['default_event_category']) : '';
    $instance['default_event_page'] = (! empty($new_instance['default_event_page'])) ? strip_tags($new_instance['default_event_page']) : '';
    return $instance;
  }

  function events_defaults() {
    return array(
      'title'                   => 'Upcoming Events',
      'events_count'            => 5,
      'event_category_1'        => -1,
      'event_page_1'            => "",
      'event_category_2'        => -1,
      'event_page_2'            => "",
      'default_event_category'  => -1,
      'default_event_page'      => ""
    );
  }

} // class My_Widget

add_action('widgets_init', function(){
     register_widget('Omnistream_Events');
});
