<?php
/*
 * Omnistream Subscribe Widget Class
*/
// Block direct requests
if (!defined('ABSPATH')) {
  die('-1');
}
/**
 * Adds Omnistream_Subscribe widget.
 */
class Omnistream_Subscribe extends WP_Widget {
  /**
   * Register widget with WordPress.
   */
  function __construct() {
    parent::__construct(
      'omnistream_subscribe', // Base ID
      __('Omnistream Subscribe Widget', 'text_domain'), // Name
      array('description' => __('Newsletter signup through Omnistream', 'text_domain'),) // Args
    );
  }
  /**
   * Front-end display of widget.
   *
   * @see WP_Widget::widget()
   *
   * @param array $args     Widget arguments.
   * @param array $instance Saved values from database.
   */
  public function widget($args, $instance) {

    // fill in defaults for unset fields
    $instance = wp_parse_args((array) $instance, $this->subscribe_defaults());

    extract($args);

    $title = apply_filters('widget_title', $instance['title']);

       echo $before_widget;

    if ($title) {
      echo $before_title . $title . $after_title;
    }

    $description_string = $instance['description_string'];

    wp_enqueue_script('omnistream-newsletter-js');

    echo <<<WIDGET
    <div class="omnistream-subscribe">
      <div class="description">$description_string</div>
      <form>
        <div>
          <input id="email" type="email" name="email" size="40" class="email" aria-required="true" aria-invalid="false" placeholder="Email Address" /><button>Sign up</button>
        </div>
        <div>
          <label id="email-error" class="error" for="email">
            This field is required.
          </label>
          <label id="submission-error" class="error" for="email">
          </label>
        </div>
      </form>
      <div class="mail-sent-ok" role="alert">
        Your email has been recorded. Thanks.
      </div>
    </div>
WIDGET;

    echo $after_widget;
  }
  /**
   * Back-end widget form.
   *
   * @see WP_Widget::form()
   *
   * @param array $instance Previously saved values from database.
   */
  public function form($instance) {

    // Set default values
    $instance = wp_parse_args((array) $instance, $this->subscribe_defaults());

    $title = esc_attr($instance['title']);
    $description_string = esc_attr($instance['description_string']);
    ?>
    <p>
      <label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?></label>
      <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>">
    </p>
    <p>
      <label for="<?php echo $this->get_field_id('description_string'); ?>"><?php _e('Description:'); ?></label>
      <input class="widefat" id="<?php echo $this->get_field_id('description_string'); ?>" name="<?php echo $this->get_field_name('description_string'); ?>" type="text" value="<?php echo esc_attr($description_string); ?>">
    </p>
    <?php
  }
  /**
   * Sanitize widget form values as they are saved.
   *
   * @see WP_Widget::update()
   *
   * @param array $new_instance Values just sent to be saved.
   * @param array $old_instance Previously saved values from database.
   *
   * @return array Updated safe values to be saved.
   */
  public function update($new_instance, $old_instance) {
    $instance = array();
    $instance['title'] = (! empty($new_instance['title'])) ? strip_tags($new_instance['title']) : '';
    $instance['description_string'] = (! empty($new_instance['description_string'])) ? strip_tags($new_instance['description_string']) : '';
    return $instance;
  }

  function subscribe_defaults() {
    return array(
      'title' => 'Newsletter Signup!',
      'description_string' => 'Sign up for news and updates'
    );
  }
} // class My_Widget

add_action('widgets_init', function(){
     register_widget('Omnistream_Subscribe');
});
